/**
 * Copyright © 2024 IAV GmbH Ingenieurgesellschaft Auto und Verkehr, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

import React, { useContext } from "react";
import { ContextMenu } from "primereact/contextmenu";
import { LanguageContext } from "../../contexts/language";
import { useTranslator } from "../internationalization/translators";
import { RadioButton } from "primereact/radiobutton";
import { ColorSettingsContext } from "../../contexts/colorsettings";

// ##############################################
// Notice: The enclosed imports are copied from 'primereact/menuitem/MenuItem' as the path could not be resolved by the gitlab builder
// although local builds were running fine - this should be removed as soon as a fixed version of primereact is published

//eslint-disable-next-line
export const SettingsMenu = /*#__PURE__*/React.forwardRef((props, ref) => {
  const colorSettingsContext = useContext(ColorSettingsContext);
  const langContext = useContext(LanguageContext);
  const t = useTranslator();
  const basicOptions = [];
  let notFallbackLang = false;
  if (!props.menuOptions?.hideLanguageSelection) {
    // Marking active language in language selection.
    // Check whether translations for the user defined language exist. Otherwise the fallback language is displayed as active.
    if (langContext) {
      const languageOptions = [];
      Object.keys(langContext.resources).forEach(key => {
        if (key !== langContext?.fallbackLang) {
          // Has to check whether the active language and key are equal or if the active language is a dialect of the language of key and the
          // resources don't contain the active language.
          // The active language could be "de-De" and the language of key could be "de". So it isn't sufficient to check whether the active
          // language is equal to key.
          const activeLang = langContext?.activeLang.replaceAll("-", "_");
          const active = activeLang === key || isDialectOf(activeLang, key) && !containsLanguage(activeLang, langContext?.resources);
          languageOptions.push({
            label: langContext?.resources[key].translation.option_name,
            icon: active ? "pi pi-check" : "",
            // I18Next must have the representation with "-" instead of "_". "de_DE" will not resolve to "de" which is necessary
            // in case translations for "de_DE" don't exist.
            command: () => {
              langContext?.selectLanguage(key.replaceAll("_", "-"));
            }
          });
          if (active) {
            notFallbackLang = active;
          }
        }
      });
      languageOptions.push({
        label: langContext?.resources[langContext.fallbackLang].translation.option_name,
        icon: !notFallbackLang ? "pi pi-check" : "",
        command: () => langContext?.selectLanguage(langContext.fallbackLang)
      });
      basicOptions.push({
        label: t("Language"),
        icon: "pi pi-comment",
        items: languageOptions.sort((option1, option2) => option1.label === option2.label ? 0 : option1.label < option2.label ? -1 : 1)
      });
    }
  }
  if (!props.menuOptions?.hideColorThemeToggler) {
    const colorSetting = colorSettingsContext?.darkmode ? "bg-grey-5 color-white" : " bg-white-1 color-black";
    basicOptions.push({
      template: /*#__PURE__*/React.createElement("div", {
        className: colorSetting + " flex justify-content-center mt-2 mb-2"
      }, /*#__PURE__*/React.createElement("div", {
        className: "flex align-items-center"
      }, /*#__PURE__*/React.createElement(RadioButton, {
        inputId: "lightMode",
        value: false,
        onChange: e => colorSettingsContext?.setDarkmode(e.value),
        checked: !colorSettingsContext?.darkmode
      }), /*#__PURE__*/React.createElement("label", {
        htmlFor: "ingredient1",
        className: "ml-1 mr-2"
      }, "Lightmode")), /*#__PURE__*/React.createElement("div", {
        className: "flex align-items-center"
      }, /*#__PURE__*/React.createElement(RadioButton, {
        inputId: "darkMode",
        value: true,
        onChange: e => colorSettingsContext?.setDarkmode(e.value),
        checked: colorSettingsContext?.darkmode
      }), /*#__PURE__*/React.createElement("label", {
        htmlFor: "ingredient2",
        className: "ml-1"
      }, "Darkmode")))
    });
  }
  const model = props.menuOptions?.additionalItems ? [...props.menuOptions.additionalItems, ...basicOptions] : basicOptions;
  return /*#__PURE__*/React.createElement("div", {
    onKeyDown: e => props.hideMenu(e)
  }, /*#__PURE__*/React.createElement(ContextMenu, {
    style: {
      width: "14.5rem",
      padding: "0.25rem 1rem 0.25rem 1rem"
    },
    ref: ref,
    model: model
  }));
});

// Checks whether "dialect" is a dialect of "baseLang".
function isDialectOf(dialect, baseLang) {
  const baseLangOfDialect = dialect.split("_")[0];
  return baseLang === baseLangOfDialect;
}

// Checks whether "resources" contain the language key "lang".
function containsLanguage(lang, resources) {
  const dialects = Object.keys(resources).filter(key => key === lang);
  return dialects.length === 1;
}